﻿using System;
using System.Text;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.Devices.Sensors;
using Windows.Storage.Streams;
using Windows.Networking.Sockets;
using Windows.Devices.Enumeration;
using Windows.Devices.Bluetooth.Rfcomm;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=391641

namespace PhoneLogic
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        Accelerometer accelerometer;
        private RfcommDeviceService rfcommService;
        private StreamSocket socket;
        private DataWriter writer;
        private char cmdSent = ' ';
        public MainPage()
        {
            this.InitializeComponent();

            this.NavigationCacheMode = NavigationCacheMode.Required;

            accelerometer = Accelerometer.GetDefault();
            uint desiredReportInterval;

            App.Current.Suspending += App_Suspending;

            if (accelerometer != null)
            {

                uint minReportInterval = accelerometer.MinimumReportInterval;
                desiredReportInterval = minReportInterval > 250 ? minReportInterval : 250;
                // slowing this way down to prevent flooding with commands
                accelerometer.ReportInterval = desiredReportInterval;
                //add event for accelerometer readings
                accelerometer.ReadingChanged += new TypedEventHandler<Accelerometer, AccelerometerReadingChangedEventArgs>(ReadingChanged);
            }
            else
            {
                MessageDialog ms = new MessageDialog("No accelerometer Found");
                ms.ShowAsync();
            }
        }
        private void App_Suspending(object sender, Windows.ApplicationModel.SuspendingEventArgs e)
        {
            Disconnect();
        }
        private async void EnumerateButton_Click(object sender, RoutedEventArgs e)
        {
            EnumerateButton.IsEnabled = false;

            try
            {
                var serviceInfoCollection =
                    await DeviceInformation.FindAllAsync(
                        RfcommDeviceService.GetDeviceSelector(RfcommServiceId.SerialPort));

                ServicesList.Items.Clear();

                if (serviceInfoCollection.Count > 0)
                {
                    foreach (var serviceInfo in serviceInfoCollection)
                    {
                        ServicesList.Items.Add(serviceInfo);
                    }
                    ServicesList.Visibility = Visibility.Visible;
                }
            }
            catch (Exception ex)
            {
                NotifyStatus(ex.Message);
            }

            EnumerateButton.IsEnabled = true;
        }

        private async void ServicesList_Tapped(object sender, Windows.UI.Xaml.Input.TappedRoutedEventArgs e)
        {
            ServicesList.IsEnabled = false;

            var serviceInfo = (DeviceInformation)ServicesList.SelectedItem;
            try
            {
                // Initialize the target Bluetooth RFCOMM device service
                rfcommService = await RfcommDeviceService.FromIdAsync(serviceInfo.Id);

                if (rfcommService != null)
                {
                    // Create a socket and connect to the target 
                    socket = new StreamSocket();
                    await socket.ConnectAsync(
                        rfcommService.ConnectionHostName,
                        rfcommService.ConnectionServiceName,
                        SocketProtectionLevel.BluetoothEncryptionAllowNullAuthentication);

                    writer = new DataWriter(socket.OutputStream);

                    EnumerateButton.IsEnabled = false;
                    ServicesList.Visibility = Visibility.Collapsed;
                    DisconnectButton.IsEnabled = true;
                }
            }
            catch (Exception ex)
            {
                NotifyStatus(ex.Message);
            }

            ServicesList.IsEnabled = true;
        }

        /// <summary>
        /// Disconnect from the currently connected radio.
        /// </summary>
        private void DisconnectButton_Click(object sender, RoutedEventArgs e)
        {
            Disconnect();
        }

        /// <summary>
        /// Cleanup Bluetooth resources
        /// </summary>
        private void Disconnect()
        {
            if (writer != null)
            {
                writer.DetachStream();
                writer = null;
            }

            if (socket != null)
            {
                socket.Dispose();
                socket = null;
            }

            if (rfcommService != null)
            {
                rfcommService = null;
            }

            EnumerateButton.IsEnabled = true;
            ServicesList.Visibility = Visibility.Collapsed;
            DisconnectButton.IsEnabled = false;
        }

        async void NotifyStatus(string status)
        {
            await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
            {
                StatusText.Text = status;
            });
        }

        private async void ReadingChanged(Accelerometer sender, AccelerometerReadingChangedEventArgs args)
        {
            byte[] packet;
            await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
            {
                if (DisconnectButton.IsEnabled == true)
                {
                    AccelerometerReading reading = args.Reading;
                    if (reading.AccelerationX > .4)  //these are arbitrary levels set to what seemed easy
                    {
                        // Send Right
                        if (cmdSent != 'R')
                        {
                            txtRight.Text = "On";
                            txtLeft.Text = " L";
                            txtForward.Text = " F";
                            txtBack.Text = " B";
                            packet = Encoding.UTF8.GetBytes("R");
                            cmdSent = 'R';
                            SendData(packet);
                        }
                    }
                    else if (reading.AccelerationX < -.4)
                    {
                        //send left
                        if (cmdSent != 'L')
                        {
                            txtRight.Text = " R";
                            txtLeft.Text = "On";
                            txtForward.Text = " F";
                            txtBack.Text = " B";
                            packet = Encoding.UTF8.GetBytes("L");
                            cmdSent = 'L';
                            SendData(packet);
                        }
                    }
                    else if (reading.AccelerationY < -.25)
                    {
                        //send back
                        if (cmdSent != 'B')
                        {
                            txtRight.Text = " R";
                            txtLeft.Text = " L";
                            txtForward.Text = " F";
                            txtBack.Text = "On";
                            packet = Encoding.UTF8.GetBytes("B");
                            cmdSent = 'B';
                            SendData(packet);
                        }
                    }
                    else if (reading.AccelerationY > .35)
                    {
                        //send Forward
                        if (cmdSent != 'F')
                        {
                            txtRight.Text = " R";
                            txtLeft.Text = " L";
                            txtForward.Text = "On";
                            txtBack.Text = " B";
                            packet = Encoding.UTF8.GetBytes("F");
                            cmdSent = 'F'; ;
                            SendData(packet);
                        }
                    }
                    else
                    {
                        if (cmdSent != 'S')
                        {
                            txtRight.Text = " R";
                            txtLeft.Text = " L";
                            txtForward.Text = " F";
                            txtBack.Text = " B";
                            packet = Encoding.UTF8.GetBytes("S");
                            cmdSent = 'S';
                            SendData(packet);
                        }
                    }

                }
            });
        }

        private async void SendData(byte[] packet)
        {
            try
            {
                writer.WriteBytes(packet);
                await writer.StoreAsync();
            }
            catch (Exception ex)
            {
                NotifyStatus(ex.Message);
            }
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.
        /// This parameter is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            // TODO: Prepare page for display here.

            // TODO: If your application contains multiple pages, ensure that you are
            // handling the hardware Back button by registering for the
            // Windows.Phone.UI.Input.HardwareButtons.BackPressed event.
            // If you are using the NavigationHelper provided by some templates,
            // this event is handled for you.
        }

    }
}
